/// <reference path="../typings/main.d.ts" />
"use strict";
var lodash_1 = require('lodash');
;
;
;
function Apollo(_a) {
    var client = _a.client, queries = _a.queries, mutations = _a.mutations;
    var watchQuery = client.watchQuery, mutate = client.mutate;
    // noop by default
    queries = queries || lodash_1.noop;
    mutations = mutations || lodash_1.noop;
    // holds latest values to track changes
    var lastQueryVariables = {};
    return function (sourceTarget) {
        var target = sourceTarget;
        var oldHooks = {};
        var hooks = {
            /**
             * Initialize the component
             * after Angular initializes the data-bound input properties.
             */
            ngOnInit: function () {
                // use component's context
                handleQueries(this);
                handleMutations(this);
            },
            /**
             * Detect and act upon changes that Angular can or won't detect on its own.
             * Called every change detection run.
             */
            ngDoCheck: function () {
                // use component's context
                handleQueries(this);
                handleMutations(this);
            },
        };
        // attach hooks
        lodash_1.forIn(hooks, function (hook, name) {
            wrapPrototype(name, hook);
        });
        function handleQueries(component) {
            lodash_1.forIn(queries(component), function (_a, queryName) {
                var query = _a.query, variables = _a.variables;
                if (!equalVariablesOf(queryName, variables)) {
                    createQuery(component, queryName, { query: query, variables: variables });
                }
            });
        }
        function handleMutations(component) {
            lodash_1.forIn(mutations(component), function (method, mutationName) {
                createMutation(component, mutationName, method);
            });
        }
        /**
         * Assings WatchQueryHandle to the component
         *
         * @param  {any}    component   Component's context
         * @param  {string} queryName   Query's name
         * @param  {Object} options     Query's options
         */
        function createQuery(component, queryName, _a) {
            var query = _a.query, variables = _a.variables;
            // save variables so they can be used in futher comparasion
            lastQueryVariables[queryName] = variables;
            // assign to component's context
            component[queryName] = watchQuery({ query: query, variables: variables });
        }
        /**
         * Assings wrapper of mutation to the component
         *
         * @param  {any}      component    Component's context
         * @param  {string}   mutationName Mutation's name
         * @param  {Function} method       Method returning mutation options
         * @return {Promise}               Mutation result
         */
        function createMutation(component, mutationName, method) {
            // assign to component's context
            component[mutationName] = function () {
                var args = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    args[_i - 0] = arguments[_i];
                }
                var _a = method.apply(client, args), mutation = _a.mutation, variables = _a.variables;
                return mutate({ mutation: mutation, variables: variables });
            };
        }
        /**
         * Compares current variables with previous ones.
         *
         * @param  {string}  queryName  Query's name
         * @param  {any}     variables  current variables
         * @return {boolean}            comparasion result
         */
        function equalVariablesOf(queryName, variables) {
            return lodash_1.isEqual(lastQueryVariables[queryName], variables);
        }
        /**
         * Creates a new prototype method which is a wrapper function
         * that calls new function before old one.
         *
         * @param  {string}   name
         * @param  {Function} func
         */
        function wrapPrototype(name, func) {
            oldHooks[name] = sourceTarget.prototype[name];
            // create a wrapper
            target.prototype[name] = function () {
                var args = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    args[_i - 0] = arguments[_i];
                }
                // to call a new prototype method
                func.apply(this, args);
                // call the old prototype method
                if (oldHooks[name]) {
                    oldHooks[name].apply(this, args);
                }
            };
        }
        // return decorated target
        return target;
    };
}
exports.Apollo = Apollo;
//# sourceMappingURL=apolloDecorator.js.map